/*  Strobe
	by Joe Wang - www.columbia.edu/~jw168, jw168@columbia.edu, joe@timewarp.net
	created - 7/22/96
	a strobe light
	*/

#include <AppKit.h>

#define DAYTIME 10150
#define NIGHTTIME 110500

typedef long (*rect)(long,long,long,long,uchar);

static long Draw(void *win);
sem_id lock;

ulong day, night;

class Win : public BWindowScreen {
	rgb_color palette[256];
	thread_id strobe;
	ulong width;
public:
	Win() : BWindowScreen("screen", B_8_BIT_640x480) {
		day=DAYTIME;
		night=NIGHTTIME;
		palette[0].red=palette[0].green=palette[0].blue=0;
		palette[1].red=palette[1].green=palette[1].blue=255;
		for(int i=2; i<256; i++) {
			palette[i].red=palette[i].green=palette[i].blue=rand();			
			}
		SetColorList(palette);
		strobe=0;
		strobe=spawn_thread(Draw, "strobe", B_DISPLAY_PRIORITY, (void *)this);
		resume_thread(strobe);
		width = CardInfo()->width;
		}
	void ScreenConnected(bool active) {
		if (active) release_sem(lock);
    	else acquire_sem(lock);
		}
	void MessageReceived(BMessage *msg) {
		long key, cha;
		switch(msg->what) {
			case B_KEY_DOWN:
				cha=msg->FindLong("char");			
				switch(cha) {
					case '-':
						night-=100;
						break;
					case '=':
						night+=100;
						break;
					case '[':
						day-=100;
						break;
					case ']':
						day+=100;
						break;
					case '_':
						night-=10;
						break;
					case '+':
						night+=10;
						break;
					case '{':
						day-=10;
						break;
					case '}':
						day+=10;
						break;
					case 'q':
						PostMessage(B_QUIT_REQUESTED);
						break;
					}
			}
		}
	bool QuitRequested() {
		be_app->PostMessage(B_QUIT_REQUESTED);
		return TRUE;
		}
	void Quit() {
		if(strobe) {
			Disconnect();
			kill_thread(strobe);
			}
		BWindowScreen::Quit();
		}
	};

static long Draw(void *win) {
	ulong width, height;
	Win *window;
	rect fill_rect;
	bool bright=TRUE;
	window = (Win *)win;
	fill_rect=(rect)window->CardHookAt(5);
	width = 639;	//
	window->CardInfo()->width-1;
	height = 479;	//
	window->CardInfo()->height-1;
	while(TRUE) {
		acquire_sem(lock);
		if(bright) {
			(fill_rect)(0, 0, width, height, 0);
			bright=FALSE;
			snooze(night);
			}
		else {
			(fill_rect)(0,0,width,height,1);
			bright=TRUE;
			snooze(day);
			}
		release_sem(lock);
		}
	}

class App : public BApplication {
	public:
		App() : BApplication('DEMO') {
			win=new Win();
			}
		bool QuitRequested() {
			win->Quit();
			return TRUE;
			}
	private:
		Win *win;
	};

main() {
	lock=create_sem(0,"screen sem"); 
	App *app=new App();
	app->Run();
	delete app;
	delete_sem(lock);
	return 0;
	}